@@ -31,10 +31,80 @@ class CheckResult:
     tree_data: Dict[str, Dict] = field(default_factory=dict)
 
 
+def run_job_checks(checks_list, check_data, test_ntuple_path_list):
+    """
+    Run the checks listed for the given job, using data from an ntuple made earlier (by `lb-ap test`).
+
+    Returns dict of CheckResult objects
+    """
+    check_results = {}
+
+    for check in checks_list:
+        data = check_data[check]
+
+        if data["type"] == "range":
+            check_results[check] = range_check(
+                test_ntuple_path_list,
+                expression=data.get("expression"),
+                limits=data.get("limits"),
+                n_bins=data.get("n_bins"),
+                blind_ranges=data.get("blind_ranges"),
+                exp_mean=data.get("exp_mean"),
+                exp_std=data.get("exp_std"),
+                mean_tolerance=data.get("mean_tolerance"),
+                std_tolerance=data.get("std_tolerance"),
+                tree_pattern=data.get("tree_pattern"),
+            )
+        elif data["type"] == "range_bkg_subtracted":
+            check_results[check] = range_check_bkg_subtracted(
+                test_ntuple_path_list,
+                expression=data.get("expression"),
+                limits=data.get("limits"),
+                expr_for_subtraction=data.get("expr_for_subtraction"),
+                mean_sig=data.get("mean_sig"),
+                background_shift=data.get("background_shift"),
+                background_window=data.get("background_window"),
+                signal_window=data.get("signal_window"),
+                n_bins=data.get("n_bins"),
+                blind_ranges=data.get("blind_ranges"),
+                tree_pattern=data.get("tree_pattern"),
+            )
+        elif data["type"] == "range_nd":
+            check_results[check] = range_check_nd(
+                test_ntuple_path_list,
+                expressions=data.get("expressions"),
+                limits=data.get("limits"),
+                n_bins=data.get("n_bins"),
+                blind_ranges=data.get("blind_ranges"),
+                tree_pattern=data.get("tree_pattern"),
+            )
+        elif data["type"] == "num_entries":
+            check_results[check] = num_entries(
+                test_ntuple_path_list,
+                count=data.get("count"),
+                tree_pattern=data.get("tree_pattern"),
+            )
+        elif data["type"] == "num_entries_per_invpb":
+            check_results[check] = num_entries_per_invpb(
+                test_ntuple_path_list,
+                count_per_invpb=data.get("count_per_invpb"),
+                tree_pattern=data.get("tree_pattern"),
+                lumi_pattern=data.get("lumi_pattern"),
+            )
+        elif data["type"] == "branches_exist":
+            check_results[check] = branches_exist(
+                test_ntuple_path_list,
+                branches=data.get("branches"),
+                tree_pattern=data.get("tree_pattern"),
+            )
+
+    return check_results
+
+
 def num_entries(
     filepath_list,
     count,
-    tree_pattern=r"(.*/DecayTree)|(.*/MCDecayTree)",
+    tree_pattern,
 ):
     """Check that all matching TTree objects contain a minimum number of entries.
 
@@ -84,12 +154,13 @@ def range_check(
     filepath_list,
     expression,
     limits,
+    n_bins,
     blind_ranges,
-    tree_pattern=r"(.*/DecayTree)|(.*/MCDecayTree)",
-    exp_mean=0.0,
-    exp_std=0.0,
-    mean_tolerance=0.0,
-    std_tolerance=0.0,
+    tree_pattern,
+    exp_mean,
+    exp_std,
+    mean_tolerance,
+    std_tolerance,
 ):
     """Check if there is at least one entry in the TTree object with a specific
     variable falling in a pre-defined range. The histogram is then produced as output.
@@ -100,6 +171,7 @@ def range_check(
     :param filepath_list: List of paths to files to analyse
     :param expression: Name of the variable (or expression depending on varibales in the TTree) to be checked
     :param limits: Pre-defined range
+    :param n_bins: number of bins for the histogram
     :param blind_ranges: regions to be blinded in the histogram
     :param exp_mean: Expected mean value (optional)
     :param exp_std: Expected standard deviation (optional)
@@ -133,7 +205,7 @@ def range_check(
                 if key not in result.tree_data:
                     result.tree_data[key] = {}
                     axis0 = hist.axis.Regular(
-                        50, limits["min"], limits["max"], name=expression
+                        n_bins, limits["min"], limits["max"], name=expression
                     )
                     bin_centers = axis0.centers
                     h = Hist(axis0)
@@ -162,14 +234,15 @@ def range_check(
                 ]
 
                 # Apply blinding
-                if isinstance(blind_ranges, dict):
-                    # Take into account that there could be multiple regions to blind
-                    blind_ranges = [blind_ranges]
-                for blind_range in blind_ranges:
-                    lower, upper = blind_range["min"], blind_range["max"]
-                    test_array = test_array[
-                        ~((lower < test_array) & (test_array < upper))
-                    ]
+                if blind_ranges is not None:
+                    if isinstance(blind_ranges, dict):
+                        # Take into account that there could be multiple regions to blind
+                        blind_ranges = [blind_ranges]
+                    for blind_range in blind_ranges:
+                        lower, upper = blind_range["min"], blind_range["max"]
+                        test_array = test_array[
+                            ~((lower < test_array) & (test_array < upper))
+                        ]
 
                 # Fill histogram
                 result.tree_data[key]["histograms"][0].fill(test_array)
@@ -177,13 +250,14 @@ def range_check(
                 # Add to event counters
                 result.tree_data[key]["num_entries"] += test_array.size
 
-                events_in_exp_mean_region = test_array[
-                    (exp_mean - exp_std < test_array)
-                    & (test_array < exp_mean + exp_std)
-                ]
-                result.tree_data[key][
-                    "num_entries_in_mean_window"
-                ] += events_in_exp_mean_region.size
+                if exp_mean is not None and exp_std is not None:
+                    events_in_exp_mean_region = test_array[
+                        (exp_mean - exp_std < test_array)
+                        & (test_array < exp_mean + exp_std)
+                    ]
+                    result.tree_data[key][
+                        "num_entries_in_mean_window"
+                    ] += events_in_exp_mean_region.size
 
     # If no matches are found the check should be marked as failed, and can return early
     if len(result.tree_data) == 0:
@@ -217,8 +291,8 @@ def range_check(
             result.tree_data[key]["stddev"] = stddev
 
             # Apply expected mean requirement
-            delta_mean = abs(mean - exp_mean)
-            if not (exp_mean == 0.0 and mean_tolerance == 0.0):
+            if exp_mean is not None and mean_tolerance is not None:
+                delta_mean = abs(mean - exp_mean)
                 if delta_mean > mean_tolerance:
                     result.passed = False
                     result.messages += [
@@ -227,8 +301,8 @@ def range_check(
                     continue
 
             # Apply expected standard deviation requirement
-            delta_std = abs(stddev - exp_std)
-            if not (exp_std == 0.0 and std_tolerance == 0.0):
+            if exp_std is not None and std_tolerance is not None:
+                delta_std = abs(stddev - exp_std)
                 if delta_std > std_tolerance:
                     result.passed = False
                     result.messages += [
@@ -249,16 +323,18 @@ def range_check(
 
 def range_check_nd(
     filepath_list,
-    expression,
+    expressions,
     limits,
+    n_bins,
     blind_ranges,
-    tree_pattern=r"(.*/DecayTree)|(.*/MCDecayTree)",
+    tree_pattern,
 ):
     """Produce 2-dimensional histograms of variables taken from a TTree object.
 
     :param filepath_list: List of paths to files to analyse
-    :param expression: Name of the variables (or expression) to be checked.
+    :param expressions: Name of the variables (or expressions) to be checked.
     :param limits: Pre-defined ranges
+    :param n_bins: number of bins for the histogram
     :param blind_ranges: regions to be blinded in the histogram
     :param tree_pattern: A regular expression for the TTree object to check
     :returns: A CheckResult object, containing tree_data key/values:
@@ -268,7 +344,7 @@ def range_check_nd(
     result = CheckResult("range_nd", True)
 
     # Check if the number of variables matches expectations
-    length_expr = len(expression)
+    length_expr = len(expressions)
     length_limits = len(limits)
     if length_expr != 2 and length_expr != 3:
         result.messages += ["Expected two or three variables."]
@@ -297,42 +373,42 @@ def range_check_nd(
                 if key not in result.tree_data:
                     result.tree_data[key] = {}
                     result.tree_data[key]["histograms"] = []
-                    for key_i, key_j in combinations(list(expression.keys()), 2):
+                    for key_i, key_j in combinations(list(expressions.keys()), 2):
                         axis0 = hist.axis.Regular(
-                            50,
+                            n_bins[key_i],
                             limits[key_i]["min"],
                             limits[key_i]["max"],
-                            name=expression[key_i],
+                            name=expressions[key_i],
                         )
                         axis1 = hist.axis.Regular(
-                            50,
+                            n_bins[key_j],
                             limits[key_j]["min"],
                             limits[key_j]["max"],
-                            name=expression[key_j],
+                            name=expressions[key_j],
                         )
                         h = Hist(axis0, axis1)
                         result.tree_data[key]["histograms"] += [h]
                     if length_expr == 3:
                         for key_i, key_j, key_k in combinations(
-                            list(expression.keys()), 3
+                            list(expressions.keys()), 3
                         ):
                             axis0 = hist.axis.Regular(
-                                50,
+                                n_bins[key_i],
                                 limits[key_i]["min"],
                                 limits[key_i]["max"],
-                                name=expression[key_i],
+                                name=expressions[key_i],
                             )
                             axis1 = hist.axis.Regular(
-                                50,
+                                n_bins[key_j],
                                 limits[key_j]["min"],
                                 limits[key_j]["max"],
-                                name=expression[key_j],
+                                name=expressions[key_j],
                             )
                             axis2 = hist.axis.Regular(
-                                50,
+                                n_bins[key_k],
                                 limits[key_k]["min"],
                                 limits[key_k]["max"],
-                                name=expression[key_k],
+                                name=expressions[key_k],
                             )
                             h = Hist(axis0, axis1, axis2)
                         result.tree_data[key]["histograms"] += [h]
@@ -340,58 +416,65 @@ def range_check_nd(
                     result.tree_data[key]["num_entries"] = 0
 
                 values_obj = {}
-                list_expressions = list(expression.values())
-                # Check if the branch is present in the TTree or if the expression is correctly written
+                list_expressions = list(expressions.values())
+                # Check if the branch is present in the TTree or if the expressions are correctly written
                 try:
                     values_obj = obj.arrays(list_expressions, library="np")
                 except uproot.exceptions.KeyInFileError as e:
                     result.messages += [f"Missing branch in {key!r} with {e!r}"]
                     result.passed = False
                     continue
+
                 # Apply blinding
-                mask_total = []
-                if isinstance(blind_ranges, dict):
-                    # Take into account that there could be multiple regions to blind
-                    blind_ranges = [blind_ranges]
-                for blind_range in blind_ranges:
-                    indexes = []
-                    mask = numpy.ones(len(values_obj[list_expressions[0]]), dtype=bool)
-                    for ax, range_limits in blind_range.items():
-                        lower, upper = range_limits["min"], range_limits["max"]
-                        indexes = numpy.where(
-                            (
-                                (values_obj[expression[ax]] < upper)
-                                & (values_obj[expression[ax]] > lower)
-                            )
-                        )
-                        mask_tmp = numpy.zeros(
+                if blind_ranges is not None:
+                    mask_total = []
+                    if isinstance(blind_ranges, dict):
+                        # Take into account that there could be multiple regions to blind
+                        blind_ranges = [blind_ranges]
+                    for blind_range in blind_ranges:
+                        indexes = []
+                        mask = numpy.ones(
                             len(values_obj[list_expressions[0]]), dtype=bool
                         )
-                        mask_tmp[indexes] = True
-                        mask = numpy.logical_and(mask, mask_tmp)
-                    mask_total.append(mask)
-                mask_final = numpy.zeros(
-                    len(values_obj[list_expressions[0]]), dtype=bool
-                )
-                for mask in mask_total:
-                    mask_final = numpy.logical_or(mask_final, mask)
-                for expr in expression.values():
-                    values_obj[expr] = values_obj[expr][~mask_final]
+                        for ax, range_limits in blind_range.items():
+                            lower, upper = range_limits["min"], range_limits["max"]
+                            indexes = numpy.where(
+                                (
+                                    (values_obj[expressions[ax]] < upper)
+                                    & (values_obj[expressions[ax]] > lower)
+                                )
+                            )
+                            mask_tmp = numpy.zeros(
+                                len(values_obj[list_expressions[0]]), dtype=bool
+                            )
+                            mask_tmp[indexes] = True
+                            mask = numpy.logical_and(mask, mask_tmp)
+                        mask_total.append(mask)
+                    mask_final = numpy.zeros(
+                        len(values_obj[list_expressions[0]]), dtype=bool
+                    )
+                    for mask in mask_total:
+                        mask_final = numpy.logical_or(mask_final, mask)
+                    for expr in expressions.values():
+                        values_obj[expr] = values_obj[expr][~mask_final]
+
                 # Fill the histograms
-                list_keys = list(expression.keys())
+                list_keys = list(expressions.keys())
                 hist_index = 0
                 for key_i, key_j in combinations(list_keys, 2):
                     h = result.tree_data[key]["histograms"][hist_index]
-                    h.fill(values_obj[expression[key_i]], values_obj[expression[key_j]])
+                    h.fill(
+                        values_obj[expressions[key_i]], values_obj[expressions[key_j]]
+                    )
                     hist_index += 1
                 # If more than two variables are given in input, return also 3D histograms
                 if length_expr == 3:
                     for key_i, key_j, key_k in combinations(list_keys, 3):
                         h = result.tree_data[key]["histograms"][hist_index]
                         h.fill(
-                            values_obj[expression[key_i]],
-                            values_obj[expression[key_j]],
-                            values_obj[expression[key_k]],
+                            values_obj[expressions[key_i]],
+                            values_obj[expressions[key_j]],
+                            values_obj[expressions[key_k]],
                         )
                         hist_index += 1
 
@@ -431,8 +514,8 @@ def range_check_nd(
 def num_entries_per_invpb(
     filepath_list,
     count_per_invpb,
-    tree_pattern=r"(.*/DecayTree)",
-    lumi_pattern=r"(.*/LumiTuple)",
+    tree_pattern,
+    lumi_pattern,
 ):
     """Check that the matching TTree objects contain a minimum number of entries per unit luminosity (pb-1).
 
@@ -531,8 +614,9 @@ def range_check_bkg_subtracted(
     background_shift,
     background_window,
     signal_window,
+    n_bins,
     blind_ranges,
-    tree_pattern=r"(.*/DecayTree)|(.*/MCDecayTree)",
+    tree_pattern,
 ):
     """Check if there is at least one entry in the TTree object with a specific
     variable falling in a pre-defined range. The background-subtracted histogram is then produced as output.
@@ -553,6 +637,7 @@ def range_check_bkg_subtracted(
     :param background_window:  Length of the background windows (of expr_for_subtraction variable).
     :param signal_window: Length of the signal window (of expr_for_subtraction variable) used for background subtraction.
     The window is centered around the value of "mean_sig".
+    :param n_bins: number of bins for the histogram
     :param blind_ranges: regions to be blinded in the histogram
     :param tree_pattern: A regular expression for the TTree object to check
     :returns: A CheckResult object, containing tree_data key/values:
@@ -597,7 +682,7 @@ def range_check_bkg_subtracted(
 
                     # Create the histogram for the control variable used to perform background subtraction
                     axis0 = hist.axis.Regular(
-                        50,
+                        n_bins,
                         background_range_low["min"],
                         background_range_high["max"],
                         name=expr_for_subtraction,
@@ -615,7 +700,7 @@ def range_check_bkg_subtracted(
 
                     # Add signal & background region histograms (using same axis)
                     axis1 = hist.axis.Regular(
-                        50, limits["min"], limits["max"], name=expression
+                        n_bins, limits["min"], limits["max"], name=expression
                     )
                     result.tree_data[key]["histograms"].append(Hist(axis1))
                     result.tree_data[key]["histograms"].append(Hist(axis1))
@@ -683,17 +768,20 @@ def range_check_bkg_subtracted(
                         & (test_array_bkg > limits["min"])
                     )
                 ]
-                if isinstance(blind_ranges, dict):
-                    blind_ranges = [blind_ranges]
-                    # Take into account that there could be multiple regions to blind
-                for blind_range in blind_ranges:
-                    lower, upper = blind_range["min"], blind_range["max"]
-                    test_array_sig = test_array_sig[
-                        ~((lower < test_array_sig) & (test_array_sig < upper))
-                    ]
-                    test_array_bkg = test_array_bkg[
-                        ~((lower < test_array_bkg) & (test_array_bkg < upper))
-                    ]
+
+                # Apply blinding
+                if blind_ranges is not None:
+                    if isinstance(blind_ranges, dict):
+                        blind_ranges = [blind_ranges]
+                        # Take into account that there could be multiple regions to blind
+                    for blind_range in blind_ranges:
+                        lower, upper = blind_range["min"], blind_range["max"]
+                        test_array_sig = test_array_sig[
+                            ~((lower < test_array_sig) & (test_array_sig < upper))
+                        ]
+                        test_array_bkg = test_array_bkg[
+                            ~((lower < test_array_bkg) & (test_array_bkg < upper))
+                        ]
 
                 # Fill signal & background histograms
                 result.tree_data[key]["histograms"][1].fill(test_array_sig)
@@ -738,7 +826,7 @@ def range_check_bkg_subtracted(
 def branches_exist(
     filepath_list,
     branches,
-    tree_pattern=r"(.*/DecayTree)|(.*/MCDecayTree)",
+    tree_pattern,
 ):
     """Check that all matching TTree objects contain a minimum number of entries.
 
