@@ -62,7 +62,13 @@ BASE_JOB_SCHEMA = {
 INPUT_SCHEMAS = {
     "bk_query": Map({"bk_query": Str(), Opt("n_test_lfns"): Int()}),
     "job_name": Map({"job_name": Str()}),
-    "prod_id": Map({"prod_id": Str()}),
+    "transform_ids": Map(
+        {
+            "transform_ids": Seq(Int()),
+            "filetype": Regex(RE_OUTPUT_FILE_TYPE),
+            Opt("n_test_lfns"): Int(),
+        }
+    ),
 }
 DEFAULT_JOB_VALUES = {
     "automatically_configure": False,
@@ -75,6 +81,7 @@ CHECK_TYPE_SCHEMAS = {
     "range": {
         "expression": Str(),  # TODO: replace this with a regex
         "limits": Map({"min": Float(), "max": Float()}),
+        Optional("n_bins"): Int(),
         Optional("blind_ranges"): Map({"min": Float(), "max": Float()})
         | Seq(Map({"min": Float(), "max": Float()})),
         Optional("exp_mean"): Float(),
@@ -97,6 +104,13 @@ CHECK_TYPE_SCHEMAS = {
                 Optional("z"): Map({"min": Float(), "max": Float()}),
             }
         ),
+        Optional("n_bins"): Map(
+            {
+                "x": Int(),
+                "y": Int(),
+                Optional("z"): Int(),
+            }
+        ),
         Optional("blind_ranges"): Seq(
             Map(
                 {
@@ -112,6 +126,7 @@ CHECK_TYPE_SCHEMAS = {
     },
     "num_entries_per_invpb": {
         "count_per_invpb": Float(),
+        Optional("lumi_pattern"): Str(),
     },
     "range_bkg_subtracted": {
         "expression": Str(),
@@ -121,6 +136,7 @@ CHECK_TYPE_SCHEMAS = {
         "background_shift": Float(),
         "background_window": Float(),
         "signal_window": Float(),
+        Optional("n_bins"): Int(),
         Optional("blind_ranges"): Map({"min": Float(), "max": Float()})
         | Seq(Map({"min": Float(), "max": Float()})),
     },
@@ -130,15 +146,30 @@ CHECK_TYPE_SCHEMAS = {
 }
 BASE_CHECK_SCHEMA = {
     "type": Enum(list(CHECK_TYPE_SCHEMAS)),
-    "tree_pattern": Str(),
+    Optional("tree_pattern"): Str(),
 }
 BASE_CHECK_DEFAULT_VALUES = {
     "tree_pattern": r"(.*/DecayTree)|(.*/MCDecayTree)",
-    "blind_ranges": [],
-    "exp_mean": 0.0,
-    "exp_std": 0.0,
-    "mean_tolerance": 0.0,
-    "std_tolerance": 0.0,
+}
+CHECK_TYPE_DEFAULT_VALUES = {
+    "num_entries": {},
+    "range": {
+        "n_bins": 50,
+    },
+    "range_nd": {
+        "n_bins": {
+            "x": 50,
+            "y": 50,
+            "z": 50,
+        },
+    },
+    "num_entries_per_invpb": {
+        "lumi_pattern": r"(.*/LumiTuple)",
+    },
+    "range_bkg_subtracted": {
+        "n_bins": 50,
+    },
+    "branches_exist": {},
 }
 
 
@@ -273,22 +304,41 @@ def parse_yaml(rendered_yaml):
     if "checks" in data1:
         # apply the appropriate schema to each different type of check
         for _check_name, check_data in data1["checks"].items():
+            check_type = str(check_data["type"])
             check_schema = {
                 **BASE_CHECK_SCHEMA,
-                **CHECK_TYPE_SCHEMAS[str(check_data["type"])],
+                **CHECK_TYPE_SCHEMAS[check_type],
             }
-            # apply default values
-            for key, value in BASE_CHECK_SCHEMA.items():
-                if key not in check_data and key in BASE_CHECK_DEFAULT_VALUES:
-                    check_schema.pop(key, None)
-                    check_schema[
-                        Optional(key, default=BASE_CHECK_DEFAULT_VALUES[key])
-                    ] = value
+            check_defaults = {
+                **BASE_CHECK_DEFAULT_VALUES,
+                **CHECK_TYPE_DEFAULT_VALUES[check_type],
+            }
+
+            # apply default values common to all check types
+            for key, value in check_schema.copy().items():
+                if isinstance(key, Optional):
+                    key = key.key
+                    if key not in check_data and key in check_defaults:
+                        key = Optional(key, default=check_defaults.get(key))
+                        check_schema[key] = value
 
             check_data.revalidate(Map(check_schema))
+
         # if checks pass validation, store elsewhere & delete from main data
         # so that normal jobs aren't impacted
         data_checks = data1.data["checks"]
+
+        # special case: range_nd defaults with variable number of axes
+        # if only 2D, remove the defaults for the 3rd axis
+        for chk, chk_data in data_checks.items():
+            if chk_data.get("type") == "range_nd":
+                if (
+                    chk_data.get("n_bins")
+                    == CHECK_TYPE_DEFAULT_VALUES["range_nd"]["n_bins"]
+                ):
+                    if len(chk_data.get("expressions")) == 2:
+                        del data_checks[chk]["n_bins"]["z"]
+
         del data1["checks"]
 
     if "defaults" in data1:
@@ -353,8 +403,9 @@ def parse_yaml(rendered_yaml):
             )
 
         # move contents of extra_checks to checks
-        data.data.setdefault("checks", [])
         data_dict = data.data
+        if "checks" not in data_dict[k]:
+            data_dict[k]["checks"] = []
         if "extra_checks" in data_dict[k]:
             data_dict[k]["checks"] += data_dict[k]["extra_checks"]
             del data_dict[k]["extra_checks"]
@@ -543,3 +594,16 @@ def _validate_checks_data(checks_data, jobs_data):
     for ck, ck_used in checks_used.items():
         if not ck_used:
             raise ValueError(f"Check {ck} is defined but not used")
+
+    # For mean/stddev in range checks: can't specify tolerance without a central value (& vice versa)
+    for _cname, cdata in checks_data.items():
+        if cdata.get("type") == "range":
+            req_pair_tuples = [
+                ("exp_mean", "mean_tolerance"),
+                ("exp_std", "std_tolerance"),
+            ]
+            for pair in req_pair_tuples + [tup[::-1] for tup in req_pair_tuples]:
+                if (cdata.get(pair[0]) is not None) and (cdata.get(pair[1])) is None:
+                    raise ValueError(
+                        f"When using '{pair[0]}', must also specify '{pair[1]}'"
+                    )
